-- A generator for immutable JAVA datastructures which simulate
-- Haskell (resp. Agda) data types
-- Author : Andreas Abel
-- Created: 2005-10-17

module Main where

import System
-- import System.Exit

import Syntax
import Lexer
import Parser
import Options
import Printer

{-
usage :: IO a
usage = do putStrLn "usage: dataGen <inputFile>"
           exitFailure

checkCmdLine :: [String] -> IO (Options, String)
checkCmdLine ["-h"] = usage
checkCmdLine ["--help"] = usage
checkCmdLine [name] = return name
checkCmdLine _ = usage
-}

header :: String -> String
header name = "// " ++ name ++ "\n// created by dataGen"
-- (C) Andreas Abel 2005\n// DO NOT EDIT!
                   ++ "\n\n"

printToFile :: Class -> IO ()
printToFile (Class id body) = 
    let name = id ++ ".java" in
    do putStrLn ("creating " ++ name)
       writeFile name $ header name ++ body ++ "\n"

sepFiles :: [Class] -> IO ()
sepFiles = foldl (\ r c -> r >> printToFile c) (return ()) 

oneFile :: Maybe String -> [Class] -> IO ()
oneFile mName cs =
    let contents = (concat $ 
                      map (\ (Class _ body) -> body ++ "\n\n") cs)
    in maybe (putStr contents) 
        (\ name ->
            do putStrLn ("creating " ++ name)   
               writeFile name $ header name ++ contents )
        mName

outputClasses :: Options -> Maybe String -> [Class] -> IO ()
outputClasses opts dest cs 
    = if (pubClasses opts) then sepFiles cs
      else oneFile dest cs

{-
dataToFile :: Data -> IO ()
dataToFile d@(Data id _) = 
    writeFile (id ++ ".java") $
      "// created by dataGen (C) Andreas Abel 2005\n// DO NOT EDIT!\n\n" ++
      renderData d
-}

fresh :: [String] -> String
fresh l = head $ filter (`notElem` l) $ map (:[]) ['A' .. 'Z']

addDefaultVisitor :: [Data] -> [Data]
addDefaultVisitor = 
    map (\ (Data id params cs vs) -> Data id params cs $ 
         let a = fresh params in
         Visitor (id ++ "Visitor") (Gen a) : vs)

main :: IO ()
main = 
    do args  <- getArgs
       (opt, src, dest) <- parseCmdLine (args)
       input <- readFile src
       let ds' = parser $ alexScanTokens $ input
       let ds = if defaultVisitor opt then addDefaultVisitor ds'
                else ds'
--       let opt = Options [Visitor "Eval" "int"]
       outputClasses opt dest $
         concat $ map (\d -> dataToClasses opt d
                             ++ dataToVisitors opt d) ds
--       foldr (\ d r -> r >> printToFile d) (return ()) $ parser $ alexScanTokens $ input


