module Options where

import Data.List (elem)
import Data.Maybe 

import System.Console.GetOpt

data Flag = Public 
          | Help 
          | Visit
          | Output String
            deriving (Eq, Show)

-- outp :: Maybe String -> Flag
-- outp = Output . fromMaybe "stdout"

optDescrs :: [OptDescr Flag]
optDescrs =
  [ Option ['h','?'] ["help"]   (NoArg Help)   "show usage information"
  , Option ['p']     ["public"] (NoArg Public) "produce public classes (many files)"
  , Option ['o']     ["output"] (ReqArg Output "FILE") "output file"
  , Option ['d']     ["defaultVisitor"] (NoArg Visit) "produce default visitor (Java 1.5)"
  ]

halt :: [String] -> IO a 
halt errs = ioError (userError (concat errs ++ usageInfo header optDescrs)) 
    where header = "usage: dataGen [OPTION...] <inputFile>" 

data Options = Options {
    pubClasses :: Bool,
    defaultVisitor :: Bool
  }

emptyOpts :: Options
emptyOpts = Options False False

sortFlags :: [Flag] -> (Options, Maybe String) -> IO (Options, Maybe String)

sortFlags [] r = return r

sortFlags (Visit:flags) (opts, s) 
    = sortFlags flags (opts{ defaultVisitor = True }, s)

sortFlags (Public:flags) (opts, s@Nothing) 
    = sortFlags flags (opts{ pubClasses = True }, s)

sortFlags (Output s:flags) (opts, Nothing) | not (pubClasses opts)
    = sortFlags flags (opts, Just s)

sortFlags _ _ = halt ["cannot use both -o and -p\n"]

parseCmdLine :: [String] -> IO (Options, String, Maybe String)
parseCmdLine argv =
    let (os, ns, errs) = getOpt Permute optDescrs argv in
    if Help `elem` os then halt []
    else case (ns, errs) of
             ([n], []) -> do (opts, dest) <- sortFlags os (emptyOpts, Nothing)
                             return (opts, n, dest)
             _         -> halt errs
