module Printer where

import Text.PrettyPrint
import Data.Char (toLower)

import Syntax 
import Options

-- type ClassId = String -- now in Options
type ClassDef = String

data Class = Class ClassId ClassDef

ind = 4

dot      = text "."
public   = text "public"
interface= text "interface"
abstract = text "abstract"
extends  = text "extends"
this     = text "this"
accept   = text "accept"
v        = text "v"
visit    = text "visit"
-- return   = text "return" -- already in use

condsep :: Bool -> Doc -> Doc -> Doc
condsep True d1 d2 = d1 <+> d2
condsep False _ d2 = d2

jApp :: Doc -> [Doc] -> Doc
jApp d1 [] = d1
jApp d1 ds = d1 <> text "<" <> cat (punctuate comma ds) <> text ">"

printType :: Type -> Doc
printType t = printType' t []

printType' :: Type -> [Type] -> Doc
printType' (List t)  ts = jApp (text "List") $ map printType $ t:ts
printType' (App r s) ts = printType' r $ s:ts
printType' (Name x)  ts = jApp (text x) $ map printType $ ts

flatReturnType :: TypeId -> Doc
flatReturnType (TypeId s) = text s
flatReturnType (Gen s)    = text s

dataToVisitors :: Options -> Data -> [Class]
dataToVisitors opt (Data id params cs vs) =
  map (constrToVisitor opt id params cs) vs

constrToVisitor :: Options -> DataId -> [Param] -> [Constructor] -> Visitor -> Class
constrToVisitor opt id params cs (Visitor name rt) = 
  Class id (render $
    condsep (pubClasses opt) public $
    interface <+> jApp (text name) (genReturnType rt ++ map text params) <+> lbrace
    $+$
    (nest ind $ vcat $ 
       map (\ (Constructor c _) -> 
            public <+> flatReturnType rt <+> visit <+> 
            parens (jApp (text c) (map text params) <+> var) <> semi)
       cs)
    $+$ rbrace)
      where var = char $ toLower (head id)

quantReturnType :: TypeId -> Doc
quantReturnType (TypeId s) = text s
quantReturnType (Gen s) = text "<" <> text s <> text ">" 
                          <+> text s

genReturnType :: TypeId -> [Doc]
genReturnType (Gen s) = [text s]
genReturnType (TypeId s) = []

dataToClasses :: Options -> Data -> [Class]
dataToClasses opt (Data id params cs vs) = 
  Class id (render $ 
        publicOpt $ abstract <+> 
            text "class" <+> jApp (text id) (map text params) <+> lbrace
        $+$ (nest ind $ vcat $ 
{-
             map (\ (Visitor name returnType) ->
                    public <+> text returnType <+> accept <+>
                    parens (text name <+> v) <+> lbrace
                    $+$
                    (nest ind $ condsep (returnType /= "void") (text "return") 
                      $ v <> dot <> visit <+> (parens this) <> semi)
                    $+$ rbrace) vs)
-}
             map (\ (Visitor name rt) -> 
                  public <+> abstract <+> quantReturnType rt <+> 
                  accept <+> parens (jApp (text name) (genReturnType rt ++ map text params) <+> v) <> semi) 
               vs)
        $+$ rbrace)
    : map (constrToClass opt id params vs) cs
      where publicOpt doc | pubClasses opt = public <+> doc
                          | otherwise = doc

constrToClass :: Options -> DataId -> [Param] -> [Visitor] -> Constructor -> Class
constrToClass opt super params vs (Constructor id fs) = 
  Class id $ render $
  (publicOpt $ text "class" <+> jApp (text id) (map text params) <+> 
    extends <+> jApp (text super) (map text params) <+> lbrace)
  $+$ (nest ind $
       (vcat (map (\ (Field f t) -> public <+> printType t <+> text f <> semi) fs))
       $+$
       (public <+> text id <+> 
        parens (cat (punctuate (comma<>space) (map (\ (Field f t) -> printType t <+> text f) fs))) <+> 
        lbrace
        $+$ (nest ind (vcat (map (\ (Field f t) -> this <> dot <> text f <+> equals <+> text f <> semi) fs)))
        $+$ rbrace)
        $+$
         (vcat $ map (\ (Visitor name rt) ->
                    public <+> quantReturnType rt <+> accept <+>
                    parens (jApp (text name) (genReturnType rt ++ map text params) <+> v) <+> lbrace
                    $+$
                    (nest ind $ condsep (rt /= TypeId "void") (text "return") 
                      $ v <> dot <> visit <+> (parens this) <> semi)
                    $+$ rbrace) vs)
{-
       $+$
       (vcat $ map (\ (Visitor name returnType) ->
                    public <+> text returnType <+> accept <+>
                    parens (text name <+> v) <+> lbrace
                    $+$
                    (nest ind $ text "return" <+> v <> dot <> visit <+>
                     (parens this) <> semi)
                    $+$ rbrace)
                   (visitors opt))
-}
       )
  $+$ rbrace
      where publicOpt doc | pubClasses opt = public <+> doc
                          | otherwise = doc

